/*
  Name:      SD_Update.ino
  Created:   12.09.2017 15:07:17
  Author:    Frederik Merz <frederik.merz@novalight.de>
  Purpose:   Update firmware from SD card
  Steps:
   1. Flash this image to the ESP32 an run it
   2. Copy update.bin to a SD-Card, you can basically
      compile this or any other example
      then copy and rename the app binary to the sd card root
   3. Connect SD-Card as shown in SD_MMC example,
      this can also be adapted for SPI
   3. After successfull update and reboot, ESP32 shall start the new app
*/
#include <Update.h>
#include "FS.h"
#include "SD.h"
#include "SPI.h"

// perform the actual update from a given stream
void performUpdate(Stream &updateSource, size_t updateSize) {
  if (Update.begin(updateSize)) {
    size_t written = Update.writeStream(updateSource);
    if (written == updateSize) {
      // Serial.println("Written : " + String(written) + " successfully");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Written : " + String(written) + " successfully");
      display.display();
    }
    else {
      // Serial.println("Written only : " + String(written) + "/" + String(updateSize) + ". Retry?");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Written only : " + String(written) + "/" + String(updateSize) + ". Retry?");
      display.display();
    }
    if (Update.end()) {
      // Serial.println("OTA done!");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("OTA done!");
      display.display();
      if (Update.isFinished()) {
        // Serial.println("Update successfully completed. Rebooting.");
        display.clearDisplay();
        display.setCursor(0, 0);            // Start at top-left corner
        display.println("Update successfully completed. Rebooting.");
        display.display();
      }
      else {
        // Serial.println("Update not finished? Something went wrong!");
        display.clearDisplay();
        display.setCursor(0, 0);            // Start at top-left corner
        display.println(F("Update not finished? Something went wrong!"));
        display.display();
      }
    }
    else {
      // Serial.println("Error Occurred. Error #: " + String(Update.getError()));
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Error Occurred. Error #: " + String(Update.getError()));
      display.display();
    }
  }
  else
  {
    // Serial.println("Not enough space to begin OTA");
    display.clearDisplay();
    display.setCursor(0, 0);            // Start at top-left corner
    display.println("Not enough space to begin OTA");
    display.display();
  }
}

// check given FS for valid update.bin and perform update if available
void updateFromFS(fs::FS &fs) {
  File updateBin = fs.open("/update.bin");
  if (updateBin) {
    if (updateBin.isDirectory()) {
      // Serial.println("Error, update.bin is not a file");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Error, update.bin is not a file");
      display.display();
      updateBin.close();
      return;
    }
    size_t updateSize = updateBin.size();
    if (updateSize > 0) {
      // Serial.println("Try to start update");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Try to start update");
      display.display();
      performUpdate(updateBin, updateSize); // команда обновления
    }
    else {
      // Serial.println("Error, file is empty");
      display.clearDisplay();
      display.setCursor(0, 0);            // Start at top-left corner
      display.println("Error, file is empty");
      display.display();
    }
    updateBin.close();
    // whe finished remove the binary from sd card to indicate end of the process
    fs.remove("/update.bin");
  }
  else {
    // Serial.println("Could not load update.bin from sd root");
    display.clearDisplay();
    display.setCursor(0, 0);            // Start at top-left corner
    display.println("Could not load update.bin from sd root");
    display.display();
  }
}

void rebootEspWithReason(String reason) {
  // Serial.println(reason);
  display.clearDisplay();
  display.setCursor(0, 0);            // Start at top-left corner
  display.println(reason);
  display.display();
  delay(1000);
  ESP.restart();
}
